@@ -209,6 +209,7 @@ namespace Sass {
 
   bool Simple_Selector::operator== (const Simple_Selector& rhs) const
   {
+    if (const Pseudo_Selector* lp = dynamic_cast<const Pseudo_Selector*>(this)) return *lp == rhs;
     if (const Wrapped_Selector* lw = dynamic_cast<const Wrapped_Selector*>(this)) return *lw == rhs;
     if (const Attribute_Selector* la = dynamic_cast<const Attribute_Selector*>(this)) return *la == rhs;
     if (is_ns_eq(ns(), rhs.ns()))
@@ -218,6 +219,7 @@ namespace Sass {
 
   bool Simple_Selector::operator< (const Simple_Selector& rhs) const
   {
+    if (const Pseudo_Selector* lp = dynamic_cast<const Pseudo_Selector*>(this)) return *lp == rhs;
     if (const Wrapped_Selector* lw = dynamic_cast<const Wrapped_Selector*>(this)) return *lw < rhs;
     if (const Attribute_Selector* la = dynamic_cast<const Attribute_Selector*>(this)) return *la < rhs;
     if (is_ns_eq(ns(), rhs.ns()))
@@ -481,6 +483,49 @@ namespace Sass {
     return ns() == rhs.ns();
   }
 
+  bool Pseudo_Selector::operator== (const Pseudo_Selector& rhs) const
+  {
+    if (is_ns_eq(ns(), rhs.ns()) && name() == rhs.name())
+    {
+      Expression* lhs_ex = expression();
+      Expression* rhs_ex = rhs.expression();
+      if (rhs_ex && lhs_ex) return *lhs_ex == *rhs_ex;
+      else return lhs_ex == rhs_ex;
+    }
+    else return false;
+  }
+
+  bool Pseudo_Selector::operator== (const Simple_Selector& rhs) const
+  {
+    if (const Pseudo_Selector* w = dynamic_cast<const Pseudo_Selector*>(&rhs))
+    {
+      return *this == *w;
+    }
+    if (is_ns_eq(ns(), rhs.ns()))
+    { return name() == rhs.name(); }
+    return ns() == rhs.ns();
+  }
+
+  bool Pseudo_Selector::operator< (const Pseudo_Selector& rhs) const
+  {
+    if (is_ns_eq(ns(), rhs.ns()) && name() == rhs.name())
+    { return *(expression()) < *(rhs.expression()); }
+    if (is_ns_eq(ns(), rhs.ns()))
+    { return name() < rhs.name(); }
+    return ns() < rhs.ns();
+  }
+
+  bool Pseudo_Selector::operator< (const Simple_Selector& rhs) const
+  {
+    if (const Pseudo_Selector* w = dynamic_cast<const Pseudo_Selector*>(&rhs))
+    {
+      return *this < *w;
+    }
+    if (is_ns_eq(ns(), rhs.ns()))
+    { return name() < rhs.name(); }
+    return ns() < rhs.ns();
+  }
+
   bool Wrapped_Selector::operator== (const Wrapped_Selector& rhs) const
   {
     if (is_ns_eq(ns(), rhs.ns()) && name() == rhs.name())
@@ -1230,10 +1275,12 @@ namespace Sass {
       if ((*this)[i]->head()->is_empty_reference()) {
         // simply move to the next tail if we have "no" combinator
         if ((*this)[i]->combinator() == Complex_Selector::ANCESTOR_OF) {
-          if ((*this)[i]->tail() && (*this)[i]->has_line_feed()) {
-            (*this)[i]->tail()->has_line_feed(true);
+          if ((*this)[i]->tail() != NULL) {
+            if ((*this)[i]->has_line_feed()) {
+              (*this)[i]->tail()->has_line_feed(true);
+            }
+            (*this)[i] = (*this)[i]->tail();
           }
-          (*this)[i] = (*this)[i]->tail();
         }
         // otherwise remove the first item from head
         else {
@@ -1251,6 +1298,14 @@ namespace Sass {
     return false;
   }
 
+  bool Selector_Schema::has_parent_ref()
+  {
+    if (String_Schema* schema = dynamic_cast<String_Schema*>(contents())) {
+      return schema->length() > 0 && dynamic_cast<Parent_Selector*>(schema->at(0)) != NULL;
+    }
+    return false;
+  }
+
   void Selector_List::adjust_after_pushing(Complex_Selector* c)
   {
     // if (c->has_reference())   has_reference(true);
@@ -2017,6 +2072,29 @@ namespace Sass {
     return is_interpolant() || (right() && right()->is_right_interpolant());
   }
 
+  // delay binary expressions in function arguments
+  // https://github.com/sass/libsass/issues/1417
+  bool Binary_Expression::can_delay(void) const
+  {
+    bool l_delay = false;
+    bool r_delay = false;
+    if (op().operand == Sass_OP::DIV) {
+      if (Textual* tl = dynamic_cast<Textual*>(left())) {
+        l_delay = tl->type() == Textual::NUMBER ||
+                  tl->type() == Textual::DIMENSION;
+      } else {
+        l_delay = dynamic_cast<Number*>(left()) != NULL;
+      }
+      if (Textual* tr = dynamic_cast<Textual*>(right())) {
+        r_delay = tr->type() == Textual::NUMBER ||
+                  tr->type() == Textual::DIMENSION;
+      } else {
+        r_delay = dynamic_cast<Number*>(right()) != NULL;
+      }
+    }
+    return l_delay && r_delay;
+  }
+
   std::string AST_Node::to_string(Sass_Inspect_Options opt) const
   {
     Sass_Output_Options out(opt);
