@@ -30,18 +30,21 @@ namespace Sass {
     // it's easier to work with vectors
     std::vector<Env*>      env_stack;
     std::vector<Block*>    block_stack;
+    std::vector<AST_Node*> call_stack;
     std::vector<String*>   property_stack;
     std::vector<Selector_List*> selector_stack;
+    std::vector<Media_Block*> media_block_stack;
     std::vector<Backtrace*>backtrace_stack;
     bool              in_keyframes;
 
     Statement* fallback_impl(AST_Node* n);
 
+  private:
+    void expand_selector_list(Selector*, Selector_List* extender);
+
   public:
     Expand(Context&, Env*, Backtrace*);
-    virtual ~Expand() { }
-
-    using Operation<Statement*>::operator();
+    ~Expand() { }
 
     Statement* operator()(Block*);
     Statement* operator()(Ruleset*);
