@@ -1,11 +1,11 @@
+#include "sass.hpp"
 #include "ast.hpp"
 #include "output.hpp"
-#include "to_string.hpp"
 
 namespace Sass {
 
-  Output::Output(Context* ctx)
-  : Inspect(Emitter(ctx)),
+  Output::Output(Sass_Output_Options& opt)
+  : Inspect(Emitter(opt)),
     charset(""),
     top_nodes(0)
   {}
@@ -20,16 +20,11 @@ namespace Sass {
   void Output::operator()(Number* n)
   {
     // use values to_string facility
-    To_String to_string(ctx);
-    std::string res = n->perform(&to_string);
+    std::string res = n->to_string(opt);
     // check for a valid unit here
     // includes result for reporting
-    if (n->numerator_units().size() > 1 ||
-        n->denominator_units().size() > 0 ||
-        (n->numerator_units().size() && n->numerator_units()[0].find_first_of('/') != std::string::npos) ||
-        (n->numerator_units().size() && n->numerator_units()[0].find_first_of('*') != std::string::npos)
-    ) {
-      error(res + " isn't a valid CSS value.", n->pstate());
+    if (!n->is_valid_css_unit()) {
+      throw Exception::InvalidValue(*n);
     }
     // output the final token
     append_token(res, n);
@@ -42,15 +37,14 @@ namespace Sass {
 
   void Output::operator()(Map* m)
   {
-    To_String to_string(ctx);
-    std::string dbg(m->perform(&to_string));
+    std::string dbg(m->to_string(opt));
     error(dbg + " isn't a valid CSS value.", m->pstate());
   }
 
   OutputBuffer Output::get_buffer(void)
   {
 
-    Emitter emitter(ctx);
+    Emitter emitter(opt);
     Inspect inspect(emitter);
 
     size_t size_nodes = top_nodes.size();
@@ -60,23 +54,25 @@ namespace Sass {
     }
 
     // flush scheduled outputs
-    inspect.finalize();
+    // maybe omit semicolon if possible
+    inspect.finalize(wbuf.buffer.size() == 0);
     // prepend buffer on top
     prepend_output(inspect.output());
     // make sure we end with a linefeed
-    if (!ends_with(wbuf.buffer, ctx->linefeed)) {
+    if (!ends_with(wbuf.buffer, opt.linefeed)) {
       // if the output is not completely empty
-      if (!wbuf.buffer.empty()) append_string(ctx->linefeed);
+      if (!wbuf.buffer.empty()) append_string(opt.linefeed);
     }
 
     // search for unicode char
     for(const char& chr : wbuf.buffer) {
       // skip all ascii chars
-      if (chr >= 0) continue;
+      // static cast to unsigned to handle `char` being signed / unsigned
+      if (static_cast<unsigned>(chr) < 128) continue;
       // declare the charset
       if (output_style() != COMPRESSED)
         charset = "@charset \"UTF-8\";"
-                  + ctx->linefeed;
+                + std::string(opt.linefeed);
       else charset = "\xEF\xBB\xBF";
       // abort search
       break;
@@ -91,8 +87,7 @@ namespace Sass {
 
   void Output::operator()(Comment* c)
   {
-    To_String to_string(ctx);
-    std::string txt = c->text()->perform(&to_string);
+    std::string txt = c->text()->to_string(opt);
     // if (indentation && txt == "/**/") return;
     bool important = c->is_important();
     if (output_style() != COMPRESSED || important) {
@@ -132,10 +127,10 @@ namespace Sass {
     if (b->has_non_hoistable()) {
       decls = true;
       if (output_style() == NESTED) indentation += r->tabs();
-      if (ctx && ctx->source_comments) {
+      if (opt.source_comments) {
         std::stringstream ss;
         append_indentation();
-        ss << "/* line " << r->pstate().line+1 << ", " << r->pstate().path << " */";
+        ss << "/* line " << r->pstate().line + 1 << ", " << r->pstate().path << " */";
         append_string(ss.str());
         append_optional_linefeed();
       }
@@ -332,7 +327,8 @@ namespace Sass {
     }
     if (v) {
       append_mandatory_space();
-      v->perform(this);
+      // ruby sass bug? should use options?
+      append_token(v->to_string(/* opt */), v);
     }
     if (!b) {
       append_delimiter();
@@ -340,7 +336,8 @@ namespace Sass {
     }
 
     if (b->is_invisible() || b->length() == 0) {
-      return append_string(" {}");
+      append_optional_space();
+      return append_string("{}");
     }
 
     append_scope_opener();
