@@ -508,7 +508,7 @@ namespace Sass {
       return one_plus< alternatives< alnum,
                                      exactly<'-'>,
                                      exactly<'_'>,
-                                     exactly<'\\'> > >(src);
+                                     escape_seq > >(src);
     }
 
     const char* kwd_warn(const char* src) {
@@ -596,7 +596,7 @@ namespace Sass {
     }
     // Match CSS id names.
     const char* id_name(const char* src) {
-      return sequence<exactly<'#'>, name>(src);
+      return sequence<exactly<'#'>, identifier>(src);
     }
     // Match CSS class names.
     const char* class_name(const char* src) {
@@ -1197,8 +1197,8 @@ namespace Sass {
           >
         >,
         alternatives <
-          exactly < calc_fn_kwd >,
-          exactly < expression_kwd >,
+          word < calc_fn_kwd >,
+          word < expression_kwd >,
           sequence <
             sequence <
               exactly < progid_kwd >,
