@@ -1,88 +1,15 @@
+#include "sass.hpp"
 #include <cstdlib>
 #include <cstring>
 #include "util.hpp"
 #include "eval.hpp"
 #include "values.hpp"
 #include "sass/values.h"
+#include "sass_values.hpp"
 
 extern "C" {
   using namespace Sass;
 
-  struct Sass_Unknown {
-    enum Sass_Tag tag;
-  };
-
-  struct Sass_Boolean {
-    enum Sass_Tag tag;
-    bool          value;
-  };
-
-  struct Sass_Number {
-    enum Sass_Tag tag;
-    double        value;
-    char*         unit;
-  };
-
-  struct Sass_Color {
-    enum Sass_Tag tag;
-    double        r;
-    double        g;
-    double        b;
-    double        a;
-  };
-
-  struct Sass_String {
-    enum Sass_Tag tag;
-    bool          quoted;
-    char*         value;
-  };
-
-  struct Sass_List {
-    enum Sass_Tag       tag;
-    enum Sass_Separator separator;
-    size_t              length;
-    // null terminated "array"
-    union Sass_Value**  values;
-  };
-
-  struct Sass_Map {
-    enum Sass_Tag        tag;
-    size_t               length;
-    struct Sass_MapPair* pairs;
-  };
-
-  struct Sass_Null {
-    enum Sass_Tag tag;
-  };
-
-  struct Sass_Error {
-    enum Sass_Tag tag;
-    char*         message;
-  };
-
-  struct Sass_Warning {
-    enum Sass_Tag tag;
-    char*         message;
-  };
-
-  union Sass_Value {
-    struct Sass_Unknown unknown;
-    struct Sass_Boolean boolean;
-    struct Sass_Number  number;
-    struct Sass_Color   color;
-    struct Sass_String  string;
-    struct Sass_List    list;
-    struct Sass_Map     map;
-    struct Sass_Null    null;
-    struct Sass_Error   error;
-    struct Sass_Warning warning;
-  };
-
-  struct Sass_MapPair {
-    union Sass_Value* key;
-    union Sass_Value* value;
-  };
-
   // Return the sass tag for a generic sass value
   enum Sass_Tag ADDCALL sass_value_get_tag(const union Sass_Value* v) { return v->unknown.tag; }
 
@@ -164,7 +91,7 @@ extern "C" {
     if (v == 0) return 0;
     v->number.tag = SASS_NUMBER;
     v->number.value = val;
-    v->number.unit = unit ? sass_strdup(unit) : 0;
+    v->number.unit = unit ? sass_copy_c_string(unit) : 0;
     if (v->number.unit == 0) { free(v); return 0; }
     return v;
   }
@@ -187,7 +114,7 @@ extern "C" {
     if (v == 0) return 0;
     v->string.quoted = false;
     v->string.tag = SASS_STRING;
-    v->string.value = val ? sass_strdup(val) : 0;
+    v->string.value = val ? sass_copy_c_string(val) : 0;
     if (v->string.value == 0) { free(v); return 0; }
     return v;
   }
@@ -198,7 +125,7 @@ extern "C" {
     if (v == 0) return 0;
     v->string.quoted = true;
     v->string.tag = SASS_STRING;
-    v->string.value = val ? sass_strdup(val) : 0;
+    v->string.value = val ? sass_copy_c_string(val) : 0;
     if (v->string.value == 0) { free(v); return 0; }
     return v;
   }
@@ -239,7 +166,7 @@ extern "C" {
     union Sass_Value* v = (Sass_Value*) calloc(1, sizeof(Sass_Value));
     if (v == 0) return 0;
     v->error.tag = SASS_ERROR;
-    v->error.message = msg ? sass_strdup(msg) : 0;
+    v->error.message = msg ? sass_copy_c_string(msg) : 0;
     if (v->error.message == 0) { free(v); return 0; }
     return v;
   }
@@ -249,7 +176,7 @@ extern "C" {
     union Sass_Value* v = (Sass_Value*) calloc(1, sizeof(Sass_Value));
     if (v == 0) return 0;
     v->warning.tag = SASS_WARNING;
-    v->warning.message = msg ? sass_strdup(msg) : 0;
+    v->warning.message = msg ? sass_copy_c_string(msg) : 0;
     if (v->warning.message == 0) { free(v); return 0; }
     return v;
   }
@@ -350,7 +277,8 @@ extern "C" {
   {
     Memory_Manager mem;
     Value* val = sass_value_to_ast_node(mem, v);
-    std::string str(val->to_string(compressed, precision));
+    Sass_Inspect_Options options(compressed ? COMPRESSED : NESTED, precision);
+    std::string str(val->to_string(options));
     return sass_make_qstring(str.c_str());
   }
 
@@ -364,42 +292,43 @@ extern "C" {
 
       Value* lhs = sass_value_to_ast_node(mem, a);
       Value* rhs = sass_value_to_ast_node(mem, b);
+      struct Sass_Inspect_Options options(NESTED, 5);
 
       // see if it's a relational expression
       switch(op) {
         case Sass_OP::EQ:  return sass_make_boolean(Eval::eq(lhs, rhs));
         case Sass_OP::NEQ: return sass_make_boolean(!Eval::eq(lhs, rhs));
-        case Sass_OP::GT:  return sass_make_boolean(!Eval::lt(lhs, rhs) && !Eval::eq(lhs, rhs));
-        case Sass_OP::GTE: return sass_make_boolean(!Eval::lt(lhs, rhs));
-        case Sass_OP::LT:  return sass_make_boolean(Eval::lt(lhs, rhs));
-        case Sass_OP::LTE: return sass_make_boolean(Eval::lt(lhs, rhs) || Eval::eq(lhs, rhs));
+        case Sass_OP::GT:  return sass_make_boolean(!Eval::lt(lhs, rhs, "gt") && !Eval::eq(lhs, rhs));
+        case Sass_OP::GTE: return sass_make_boolean(!Eval::lt(lhs, rhs, "gte"));
+        case Sass_OP::LT:  return sass_make_boolean(Eval::lt(lhs, rhs, "lt"));
+        case Sass_OP::LTE: return sass_make_boolean(Eval::lt(lhs, rhs, "lte") || Eval::eq(lhs, rhs));
         default:           break;
       }
 
       if (sass_value_is_number(a) && sass_value_is_number(b)) {
         const Number* l_n = dynamic_cast<const Number*>(lhs);
         const Number* r_n = dynamic_cast<const Number*>(rhs);
-        rv = Eval::op_numbers(mem, op, *l_n, *r_n);
+        rv = Eval::op_numbers(mem, op, *l_n, *r_n, options);
       }
       else if (sass_value_is_number(a) && sass_value_is_color(a)) {
         const Number* l_n = dynamic_cast<const Number*>(lhs);
         const Color* r_c = dynamic_cast<const Color*>(rhs);
-        rv = Eval::op_number_color(mem, op, *l_n, *r_c);
+        rv = Eval::op_number_color(mem, op, *l_n, *r_c, options);
       }
       else if (sass_value_is_color(a) && sass_value_is_number(b)) {
         const Color* l_c = dynamic_cast<const Color*>(lhs);
         const Number* r_n = dynamic_cast<const Number*>(rhs);
-        rv = Eval::op_color_number(mem, op, *l_c, *r_n);
+        rv = Eval::op_color_number(mem, op, *l_c, *r_n, options);
       }
       else if (sass_value_is_color(a) && sass_value_is_color(b)) {
         const Color* l_c = dynamic_cast<const Color*>(lhs);
         const Color* r_c = dynamic_cast<const Color*>(rhs);
-        rv = Eval::op_colors(mem, op, *l_c, *r_c);
+        rv = Eval::op_colors(mem, op, *l_c, *r_c, options);
       }
       else /* convert other stuff to string and apply operation */ {
         Value* l_v = dynamic_cast<Value*>(lhs);
         Value* r_v = dynamic_cast<Value*>(rhs);
-        rv = Eval::op_strings(mem, op, *l_v, *r_v);
+        rv = Eval::op_strings(mem, op, *l_v, *r_v, options);
       }
 
       // ToDo: maybe we should should return null value?
@@ -411,8 +340,8 @@ extern "C" {
     }
 
     // simply pass the error message back to the caller for now
-    catch (Error_Invalid& e) { return sass_make_error(e.message.c_str()); }
-    catch (std::bad_alloc& ba) { return sass_make_error("memory exhausted"); }
+    catch (Exception::InvalidSass& e) { return sass_make_error(e.what()); }
+    catch (std::bad_alloc&) { return sass_make_error("memory exhausted"); }
     catch (std::exception& e) { return sass_make_error(e.what()); }
     catch (std::string& e) { return sass_make_error(e.c_str()); }
     catch (const char* e) { return sass_make_error(e); }
