@@ -6,8 +6,8 @@ module SassSpec::CLI
   def self.parse
     options = {
       engine_adapter: SassEngineAdapter.new("sass"),
-      spec_directory: "spec",
-      generate: [],
+      spec_directory: nil,
+      generate: false,
       tap: false,
       skip: false,
       verbose: false,
@@ -15,18 +15,11 @@ module SassSpec::CLI
       limit: -1,
       unexpected_pass: false,
       nuke: false,
-
-      # Constants
-      output_styles: ["nested", "compressed", "expanded", "compact"],
-      input_files: ["input.scss", "input.sass"],
-      nested_output_file: 'expected_output',
-      compressed_output_file: 'expected.compressed',
-      expanded_output_file: 'expected.expanded',
-      compact_output_file: 'expected.compact'
+      only_output_styles: []
     }
 
     OptionParser.new do |opts|
-      opts.banner = "Usage: ./sass-spec.rb [options]
+      opts.banner = "Usage: ./sass-spec.rb [options] [spec_directory...]
 
 Examples:
   Run `sassc --style compressed input.scss`:
@@ -41,6 +34,9 @@ the output matches the expected output. If you want set up your own test suite,
 follow a similar hierarchy as described in the initial comment of this script
 for your test hierarchy.
 
+This command can also be used to annotate tests to control which tests are ran
+and when. For details: ./sass-spec.rb annotate -h
+
 Make sure the command you provide prints to stdout.
 
 "
@@ -49,6 +45,14 @@ Make sure the command you provide prints to stdout.
         options[:verbose] = true
       end
 
+      opts.on("-V", "--version LANGUAGE_VERSION", "Select the Sass Language Version to test.") do |v|
+        unless SassSpec::LANGUAGE_VERSIONS.include?(v)
+          raise ArgumentError.new("Version #{v} is not valid. " +
+                                  "Did you mean one of: #{SassSpec::LANGUAGE_VERSIONS.join(', ')}")
+        end
+        options[:language_version] = v
+      end
+
       opts.on("-t", "--tap", "Output TAP compatible report") do
         options[:tap] = true
       end
@@ -57,20 +61,16 @@ Make sure the command you provide prints to stdout.
         options[:engine_adapter] = ExecutableEngineAdapater.new(v)
       end
 
-      opts.on("-g", "--generate format", "Run test and generate output files for the specified format or \"all\"") do |v|
-        if v == "all"
-          options[:generate].replace(options[:output_styles])
-        else
-          if options[:output_styles].include?(v)
-            options[:generate] << v
-          else
-            raise "--generate needs a valid output format #{options[:output_styles]} or \"all\""
-          end
-        end
+      opts.on("-g", "--generate", "Run test(s) and generate expected output file(s).") do |v|
+        options[:generate] = true
       end
 
-      opts.on("--ignore-todo", "Skip any folder named 'todo'") do
-        options[:skip_todo] = true
+      opts.on("--run-todo", "Run any tests marked as todo. Defaults to false.") do
+        options[:run_todo] = true
+      end
+
+      opts.on("--impl NAME", "Sets the name of the implementation being tested. Defaults to 'sass'") do |name|
+        options[:implementation] = name
       end
 
       opts.on("--filter PATTERN", "Run tests that match the pattern you provide") do |pattern|
@@ -81,6 +81,20 @@ Make sure the command you provide prints to stdout.
         options[:limit] = limit.to_i
       end
 
+      opts.on("--output-style STYLE", [:expanded, :compact, :nested, :compressed, :unspecified],
+              "Only run tests that have the specified output style.",
+              "Legal values: expanded, compact, nested, compressed, unspecified.") do |style|
+        style = nil if style == :unspecified
+        options[:only_output_styles] << style
+      end
+
+      opts.on("-r SPEC_DIR", "--root SPEC_DIR", "Root directory for the specs. ",
+                "Defaults to the first directory specified if not provided or ",
+                "the default spec directory if no directory is specified or if the first directory",
+                "specified is a subdirectory of the default spec directory.") do |spec_dir|
+        options[:spec_directory] = File.expand_path(spec_dir)
+      end
+
       opts.on("-s", "--skip", "Skip tests that fail to exit successfully") do
         # Note: --skip is no longer necessary as this is now the default behavior, since we test for errors
       end
@@ -98,7 +112,21 @@ Make sure the command you provide prints to stdout.
       end
     end.parse!
 
-    options[:spec_directory] = ARGV[0] if !ARGV.empty?
+    if ARGV.any? && !options[:spec_directory]
+      if File.expand_path(ARGV[0]).start_with?(SassSpec::SPEC_DIR)
+        options[:spec_directory] = SassSpec::SPEC_DIR
+      else
+        options[:spec_directory] = File.expand_path(ARGV[0])
+      end
+    end
+
+    options[:spec_directory] ||= SassSpec::SPEC_DIR
+
+    options[:spec_dirs_to_run] = ARGV.dup.map{|d| File.expand_path(d)} if ARGV.any?
+
+    if options[:implementation] && options[:engine_adapter].respond_to?(:set_description)
+      options[:engine_adapter].set_description(options[:implementation])
+    end
 
     options
   end
