@@ -1,4 +1,7 @@
+require_relative 'capture_with_timeout'
+
 class EngineAdapter
+
   def describe
     not_implemented
   end
@@ -14,7 +17,7 @@ class EngineAdapter
 
   # Compile a Sass file and return the results
   # @return [css_output, std_error, status_code]
-  def compile(sass_filename)
+  def compile(sass_filename, style, precision)
     not_implemented
   end
 
@@ -26,12 +29,18 @@ class EngineAdapter
 end
 
 class ExecutableEngineAdapater < EngineAdapter
+  include CaptureWithTimeout
 
   def initialize(command, description = nil)
     @command = command
+    @timeout = 20
     @description = description || command
   end
 
+  def set_description(description)
+    @description = description
+  end
+
   def describe
     @description
   end
@@ -44,10 +53,15 @@ class ExecutableEngineAdapater < EngineAdapter
 
 
   def compile(sass_filename, style, precision)
-    require 'open3'
-    cmd = "#{@command} --precision #{precision} -t #{style}"
-    stdout, stderr, status = Open3.capture3("#{cmd} #{sass_filename}", :binmode => true)
-    [stdout, stderr, status.exitstatus]
+    cmd = "#{@command} --precision #{precision}"
+    cmd += " -t #{style}" if style
+    result = capture3_with_timeout("#{cmd} #{sass_filename}", :binmode => true, :timeout => @timeout)
+
+    if result[:timeout]
+      ["", "Execution timed out after #{@timeout}s", -1]
+    else
+      [result[:stdout], result[:stderr], result[:status].exitstatus]
+    end
   end
 end
 
@@ -56,6 +70,10 @@ class SassEngineAdapter < EngineAdapter
     @description = description
   end
 
+  def set_description(description)
+    @description = description
+  end
+
   def describe
     @description
   end
@@ -77,7 +95,9 @@ class SassEngineAdapter < EngineAdapter
     begin
       Encoding.default_external = "UTF-8"
       Sass::Script::Value::Number.precision = precision
-      css_output = Sass.compile_file(sass_filename.to_s, :style => style.to_sym)
+      sass_options = {}
+      sass_options[:style] = style.to_sym if style
+      css_output = Sass.compile_file(sass_filename.to_s, sass_options)
       # strings come back as utf8 encoded
       # internaly we only work with bytes
       err_output = stderr.string
