@@ -20,6 +20,23 @@
 #the output of running a command on it in the file expected_output* in the same directory
 
 require_relative 'lib/sass_spec'
-unless SassSpec::Runner.new(SassSpec::CLI.parse()).run
-  exit 1
+
+if ARGV[0] == "annotate"
+  require_relative 'lib/sass_spec/annotate'
+  begin
+    (cli = SassSpec::Annotate::CLI.parse(ARGV[1..-1])) || exit(1)
+  rescue OptionParser::InvalidOption => e
+    warn e.message + "\n\n"
+    SassSpec::Annotate::CLI.parse(%w(-h))
+    exit 1
+  end
+  cli.annotate || exit(1)
+else
+  begin
+    SassSpec::Runner.new(SassSpec::CLI.parse()).run || exit(1)
+  rescue ArgumentError => e
+    warn e.message
+    warn e.backtrace
+    exit 1
+  end
 end
